<?php

namespace QUI\ERP\Payments\Stripe;

use Doctrine\DBAL\Connection;
use QUI;
use QUI\ERP\Accounting\Payments\Transactions\Handler as TransactionHandler;
use QUI\ERP\Accounting\Payments\Transactions\Transaction as EcoynTransaction;
use QUI\ERP\Order\AbstractOrder;
use QUI\ERP\Order\Handler as OrderHandler;
use Stripe\Dispute;
use Stripe\PaymentIntent;

class ChargeDisputeService
{
    public function __construct(
        private ?TransactionHandler $transactionHandler = null,
        private ?OrderHandler $orderHandler = null,
        private ?Connection $connection = null
    ) {
        if (is_null($this->transactionHandler)) {
            $this->transactionHandler = TransactionHandler::getInstance();
        }

        if (is_null($this->orderHandler)) {
            $this->orderHandler = OrderHandler::getInstance();
        }

        if (is_null($this->connection)) {
            $this->connection = QUI::getDataBaseConnection();
        }

        Provider::setupApi();
    }

    public function processDispute(Dispute $dispute): void
    {
        $order = $this->getOrderByDispute($dispute);
        Utils::sendNotificationAboutDisputedTransaction($dispute, $order);

        if ($order) {
            try {
                $order->addHistory(
                    'Stripe :: Order transaction disputed. Dispute ID: ' . $dispute->id
                );
                $order->update(QUI::getUsers()->getSystemUser());
            } catch (\Exception $exception) {
                QUI\System\Log::writeException($exception);
            }
        }

        // If the dispute is lost, mark the corresponding ecoyn transaction as erroneous.
        if ($dispute->status === Dispute::STATUS_LOST) {
            $ecoynTransaction = $this->getEcoynTransactionByDispute($dispute);

            try {
                $ecoynTransaction?->changeStatus(TransactionHandler::STATUS_ERROR);
            } catch (\Exception $exception) {
                QUI\System\Log::writeException($exception);
            }
        }
        // TODO: ggf. weiteres Vorgehen wie Transaktion rückgängig machen bzw. negative Transaktion erstellen
    }

    /**
     * @param Dispute $dispute
     * @return EcoynTransaction|null
     */
    private function getEcoynTransactionByDispute(Dispute $dispute): ?EcoynTransaction
    {
        if (!$dispute->payment_intent) {
            return null;
        }

        try {
            $paymentIntent = PaymentIntent::retrieve($dispute->payment_intent);
        } catch (\Exception $exception) {
            QUI\System\Log::writeException($exception);
            return null;
        }

        if (empty($paymentIntent->invoice)) {
            return null;
        }

        try {
            $result = $this->connection
                ->createQueryBuilder()
                ->select('quiqqer_transaction_id')
                ->from(Provider::getStripeBillingSubscriptionsTransactionsTable())
                ->where('stripe_invoice_id = :stripe_invoice_id')
                ->setParameter('stripe_invoice_id', $paymentIntent->invoice)
                ->fetchAssociative();

            if (empty($result['quiqqer_transaction_id'])) {
                return null;
            }

            return $this->transactionHandler->get($result['quiqqer_transaction_id']);
        } catch (\Exception | \Throwable $exception) {
            if ($exception instanceof \Exception) {
                QUI\System\Log::writeException($exception);
            }
            return null;
        }
    }

    /**
     * @param Dispute $dispute
     * @return AbstractOrder|null
     */
    private function getOrderByDispute(Dispute $dispute): ?AbstractOrder
    {
        if (!$dispute->payment_intent) {
            return null;
        }

        try {
            $paymentIntent = PaymentIntent::retrieve([
                'id' => $dispute->payment_intent,
                'expand' => [
                    'invoice.subscription'
                ]
            ]);
        } catch (\Exception $exception) {
            QUI\System\Log::writeException($exception);
            return null;
        }

        $metaData = $paymentIntent->metadata->toArray();
        $orderUuid = null;

        if (!empty($metaData['orderUuid'])) {
            $orderUuid = $metaData['orderUuid'];
        } elseif (!empty($paymentIntent->invoice?->subscription)) {
            $subscriptionMetaData = $paymentIntent->invoice->subscription->metadata->toArray();

            if (!empty($subscriptionMetaData['orderUuid'])) {
                $orderUuid = $subscriptionMetaData['orderUuid'];
            }
        }

        if (is_null($orderUuid)) {
            return null;
        }

        try {
            return $this->orderHandler->get($orderUuid);
        } catch (\Exception $exception) {
            QUI\System\Log::writeException($exception);
        }

        return null;
    }
}
