<?php

namespace QUI\ERP\Payments\Stripe\Cron;

use Doctrine\DBAL\Connection;
use Doctrine\DBAL\Exception;
use QUI;
use QUI\ERP\Accounting\Payments\Payments;
use QUI\ERP\Accounting\Payments\Types\Payment as PaymentType;
use QUI\ERP\Constants as ErpConstants;
use QUI\ERP\Order\Handler as OrderHandler;
use QUI\ERP\Order\Order;
use QUI\ERP\Payments\Stripe\AbstractBasePayment;
use QUI\ERP\Payments\Stripe\OrderProcessingService;
use QUI\ERP\Payments\Stripe\Provider;
use Stripe\PaymentIntent as StripePaymentIntent;

use function implode;

class PendingPaymentsService
{
    public function __construct(
        private ?Payments $payments = null,
        private ?OrderHandler $orderHandler = null,
        private ?Connection $dbConnection = null
    ) {
        if (is_null($this->payments)) {
            $this->payments = Payments::getInstance();
        }

        if (is_null($this->orderHandler)) {
            $this->orderHandler = OrderHandler::getInstance();
        }

        if (is_null($this->dbConnection)) {
            $this->dbConnection = QUI::getDataBaseConnection();
        }

        Provider::setupApi();
    }

    public static function execute(): void
    {
        $service = new PendingPaymentsService();
        $service->checkPendingOrderPayments();
    }

    /**
     * Check all pending stripe order payments.
     *
     * @return void
     * @throws Exception
     */
    public function checkPendingOrderPayments(): void
    {
        $stripePaymentTypeIds = $this->getStripePaymentTypeIds();

        // If no
        if (empty($stripePaymentTypeIds)) {
            return;
        }

        $paidStatusCodes =
            [
                ErpConstants::PAYMENT_STATUS_OPEN,
                ErpConstants::PAYMENT_STATUS_PART,
                ErpConstants::PAYMENT_STATUS_DEBIT
            ];

        // Manuay query here because query builder did not work for some reason (peat)
        $sql = "SELECT `id` FROM {$this->orderHandler->table()}";
        $sql .= " WHERE `paid_status` IN ('" . implode(',', $paidStatusCodes) . "')";
        $sql .= " AND `payment_id` IN ('" . implode("','", $stripePaymentTypeIds) . "')";
        $result = $this->dbConnection->fetchAllAssociative($sql);

        foreach ($result as $row) {
            try {
                $orderId = $row['id'];
                $order = $this->orderHandler->getOrderById($orderId);
                $this->processOrder($order);
            } catch (\Exception $exception) {
                QUI\System\Log::writeException($exception);
            }
        }
    }

    /**
     * Process an unpaid order.
     *
     * Checks if the associated Stripe PaymentIntent is (un)successful and creates/updates the order payment
     * transaction accordingly.
     *
     * @param Order $order
     * @return void
     * @throws \Stripe\Exception\ApiErrorException
     */
    private function processOrder(Order $order): void
    {
        if ($order->isPaid()) {
            return;
        }

        $stripePaymentIntentId = $order->getPaymentDataEntry(AbstractBasePayment::ATTR_STRIPE_PAYMENT_INTENT_ID);

        if (empty($stripePaymentIntentId)) {
            return;
        }

        $paymentIntent = StripePaymentIntent::retrieve($stripePaymentIntentId);

        $orderProcessingService = new OrderProcessingService($order);
        $orderProcessingService->processPaymentIntent($paymentIntent);
    }

    /**
     * @return int[]
     */
    private function getStripePaymentTypeIds(): array
    {
        // Determine payment type IDs
        $payments = $this->payments->getPayments([
            'select' => ['id'],
            'where' => [
                'payment_type' => [
                    'type' => 'IN',
                    'value' => Provider::getNonRecurringPaymentTypes()
                ]
            ]
        ]);

        $paymentTypeIds = [];

        /** @var PaymentType $Payment */
        foreach ($payments as $Payment) {
            $paymentTypeIds[] = $Payment->getId();
        }

        return $paymentTypeIds;
    }
}
