<?php

namespace QUI\ERP\Payments\Stripe;

use Exception;
use QUI;
use QUI\ERP\Accounting\Payments\Transactions\Factory as TransactionFactory;
use QUI\ERP\Accounting\Payments\Transactions\Handler as TransactionHandler;
use QUI\ERP\Accounting\Payments\Transactions\Transaction;
use QUI\ERP\Order\AbstractOrder;
use Stripe\PaymentIntent;

class OrderProcessingService implements OrderProcessingServiceInterface
{
    const ORDER_PAYMENT_ATTR_TRANSACTION_TX_ID = 'stripe_transaction_tx_id';

    public function __construct(
        private readonly AbstractOrder $order,
        private ?TransactionHandler $transactionHandler = null
    ) {
        if (is_null($this->transactionHandler)) {
            $this->transactionHandler = TransactionHandler::getInstance();
        }
    }

    /**
     * Processes a Stripe PaymentIntent for this order.
     *
     * This may set the order to SUCESSFUL or FAILED, depending on the status.
     * This also creates transactions for successful orders.
     *
     * @param PaymentIntent $paymentIntent
     * @return void
     */
    public function processPaymentIntent(PaymentIntent $paymentIntent): void
    {
        switch ($paymentIntent->status) {
            case PaymentIntent::STATUS_PROCESSING:
                $this->processPendingPaymentIntent($paymentIntent);
                break;

            case PaymentIntent::STATUS_SUCCEEDED:
                $this->processSuccessfulPaymentIntent($paymentIntent);
                break;

            case PaymentIntent::STATUS_CANCELED:
            case PaymentIntent::STATUS_REQUIRES_PAYMENT_METHOD:
                $this->processFailedPaymentIntent($paymentIntent);
                break;
        }
    }

    /**
     * Process a successful Stripe PaymentIntent
     *
     * @param PaymentIntent $paymentIntent
     * @return void
     */
    private function processPendingPaymentIntent(PaymentIntent $paymentIntent): void
    {
        try {
            if (is_null($this->getPaymentIntentTransaction())) {
                $this->createPaymentIntentTransaction($paymentIntent);
            }

            $this->order->setSuccessfulStatus();
            $this->order->setPaymentData(AbstractBasePayment::ATTR_STRIPE_ORDER_SUCCESSFUL, true);

            $this->addOrderHistoryEntry(
                'PaymentIntent transaction is PENDING. Waiting for successful status from Stripe. Order is successful.'
            );
        } catch (Exception $Exception) {
            QUI\System\Log::writeException($Exception);

            $this->addOrderHistoryEntry(
                'Error while trying to process PENDING PaymentIntent: ' . $Exception->getMessage()
            );
        }
    }

    /**
     * Process a successful Stripe PaymentIntent
     *
     * @param PaymentIntent $paymentIntent
     * @return void
     */
    private function processSuccessfulPaymentIntent(PaymentIntent $paymentIntent): void
    {
        try {
            $transaction = $this->getPaymentIntentTransaction();

            if (is_null($transaction)) {
                $transaction = $this->createPaymentIntentTransaction($paymentIntent);
            }

            $transactionAmount = (int)($transaction->getAmount() * 100);

            if ($transactionAmount !== $paymentIntent->amount_received) {
                $transaction->changeStatus(TransactionHandler::STATUS_ERROR);
                $transaction = $this->createPaymentIntentTransaction($paymentIntent);
            }

            $transaction->changeStatus(TransactionHandler::STATUS_COMPLETE);
            $this->order->setSuccessfulStatus();
            $this->order->setPaymentData(AbstractBasePayment::ATTR_STRIPE_ORDER_SUCCESSFUL, true);

            $this->addOrderHistoryEntry('PaymentIntent confirmed. Payment successful.');
        } catch (Exception $Exception) {
            QUI\System\Log::writeException($Exception);

            $this->addOrderHistoryEntry(
                'Error while trying to set order as successful: ' . $Exception->getMessage()
            );
        }
    }

    /**
     * Process a failed Stripe PaymentIntent
     *
     * @param PaymentIntent $paymentIntent
     * @return void
     */
    private function processFailedPaymentIntent(PaymentIntent $paymentIntent): void
    {
        try {
            $transaction = $this->getPaymentIntentTransaction();

            if (is_null($transaction)) {
                $transaction = $this->createPaymentIntentTransaction($paymentIntent);
            }

            $transaction->changeStatus(TransactionHandler::STATUS_ERROR);

            $this->addOrderHistoryEntry(
                "PaymentIntent transaction {$transaction->getTxId()} FAILED."
            );

            if (Provider::shouldNotifyOnFailedPayment()) {
                $lastError = $paymentIntent->last_payment_error->toArray();
                $errorMsg = '';

                if (!empty($lastError['message'])) {
                    $errorMsg = $lastError['message'];
                }

                if (!empty($lastError['type'])) {
                    $errorMsg .= ' (' . $lastError['type'] . ')';
                }

                if (!empty($lastError['decline_code'])) {
                    $errorMsg .= ' (' . $lastError['decline_code'] . ')';
                }

                Utils::sendNotificationAboutFailedOrderPayment($this->order, $errorMsg);
            }

            $this->order->setPaymentStatus(QUI\ERP\Constants::PAYMENT_STATUS_ERROR);
        } catch (Exception $Exception) {
            QUI\System\Log::writeException($Exception);

            $this->addOrderHistoryEntry(
                'Error while trying to process FAILED PaymentIntent: ' . $Exception->getMessage()
            );
        }
    }

    /**
     * Add history entry for current Order
     *
     * @param string $msg
     * @return void
     */
    private function addOrderHistoryEntry(string $msg): void
    {
        try {
            $this->order->addHistory('Stripe :: ' . $msg);
            $this->saveOrder();
        } catch (\Exception | \Throwable $exception) {
            if ($exception instanceof \Exception) {
                QUI\System\Log::writeException($exception);
            }
        }
    }

    /**
     * @param PaymentIntent $paymentIntent
     * @return Transaction
     *
     * @throws QUI\ERP\Accounting\Payments\Exception
     * @throws QUI\ERP\Accounting\Payments\Transactions\Exception
     * @throws QUI\Exception
     */
    private function createPaymentIntentTransaction(PaymentIntent $paymentIntent): Transaction
    {
        $transaction = TransactionFactory::createPaymentTransaction(
            $paymentIntent->amount / 100,
            QUI\ERP\Currency\Handler::getCurrency(mb_strtoupper($paymentIntent->currency)),
            $this->order->getUUID(),
            $this->order->getPayment()->getPaymentType()->getName(),
            [
                AbstractBasePayment::ATTR_STRIPE_PAYMENT_INTENT_ID => $paymentIntent->id
            ],
            null,
            false,
            false,
            TransactionHandler::STATUS_PENDING
        );

        $this->addOrderHistoryEntry("PaymentIntent and transaction {$transaction->getTxId()} created.");
        $this->order->setPaymentData(self::ORDER_PAYMENT_ATTR_TRANSACTION_TX_ID, $transaction->getTxId());
        $this->saveOrder();

        return $transaction;
    }

    /**
     * @return Transaction|null
     */
    private function getPaymentIntentTransaction(): ?Transaction
    {
        $transactionId = $this->order->getPaymentDataEntry(self::ORDER_PAYMENT_ATTR_TRANSACTION_TX_ID);

        if (empty($transactionId)) {
            return null;
        }

        foreach ($this->order->getTransactions() as $transaction) {
            if ($transaction->getTxId() === $transactionId) {
                return $transaction;
            }
        }

        return null;
    }

    /**
     * @return void
     * @throws QUI\Exception
     */
    private function saveOrder(): void
    {
        $this->order->update(QUI::getUsers()->getSystemUser());
    }
}
