<?php

namespace QUI\ERP\Payments\Stripe;

use QUI;
use QUI\ERP\Accounting\Invoice\Invoice;
use QUI\ERP\ErpEntityInterface;
use QUI\ERP\Exception;
use QUI\ERP\Order\AbstractOrder;
use QUI\ERP\Order\Handler as OrderHandler;
use QUI\ERP\Order\OrderInterface;
use QUI\Interfaces\Users\User as UserInterface;
use Stripe\Dispute;
use Stripe\Exception\ApiErrorException;
use Stripe\PaymentIntent;
use Stripe\PaymentMethod as StripePaymentMethod;

use function mb_strpos;
use function str_replace;

/**
 * Class Utils
 *
 * Utility class for quiqqer/payment-stripe
 */
class Utils
{
    /**
     * Get order amount in the smallest currency unit
     *
     * @param ErpEntityInterface $Source
     * @return float|int
     *
     * @throws Exception
     * @throws QUI\Exception
     * @throws StripeException
     */
    public static function getCentAmount(QUI\ERP\ErpEntityInterface $Source): float | int
    {
        if ($Source instanceof AbstractOrder) {
            $PriceCalculation = $Source->getPriceCalculation();
            $amount = $PriceCalculation->getSum()->precision(2)->get();
            return $amount * 100; // convert to the smallest currency unit
        }

        if ($Source instanceof Invoice) {
            $invoiceAmount = (float)$Source->getAttribute('toPay');
            $AmountValue = new QUI\ERP\Accounting\CalculationValue($invoiceAmount, $Source->getCurrency(), 2);
            return $AmountValue->get() * 100;
        }

        throw new StripeException('Invalid $Source provided.');
    }

    /**
     * Attaches a Stripe PaymentMethod to a Stripe Customer.
     *
     * If the Customer does not have any previous PaymentMethods, the PaymentMethod is automatically
     * set as the default.
     *
     * @param UserInterface $QuiqqerUser
     * @param string $paymentMethodId
     * @return void
     *
     * @throws StripeException
     * @throws ApiErrorException
     * @throws QUI\Exception
     */
    public static function attachPaymentMethodToStripeCustomer(
        UserInterface $QuiqqerUser,
        string $paymentMethodId
    ): void {
        // If ERPUser -> get QUIQQER User
        if ($QuiqqerUser instanceof QUI\ERP\User) {
            $QuiqqerUser = QUI::getUsers()->get($QuiqqerUser->getUUID());
        }

        $stripeCustomerId = $QuiqqerUser->getAttribute(AbstractBasePayment::USER_ATTR_STRIPE_CUSTOMER_ID);

        if (empty($stripeCustomerId)) {
            throw new StripeException([
                'quiqqer/payment-stripe',
                'exception.Utils.attachPaymentMethodToStripeCustomer.no_stripe_customer_id',
                [
                    'userId' => $QuiqqerUser->getUUID()
                ]
            ]);
        }

        Provider::setupApi();
        $StripePaymentMethod = StripePaymentMethod::retrieve($paymentMethodId);

        try {
            $StripePaymentMethod->attach([
                'customer' => $stripeCustomerId
            ]);
        } catch (ApiErrorException $Exception) {
            if (mb_strpos($Exception->getMessage(), 'has already been attached')) {
                return;
            }

            throw $Exception;
        }
    }

    /**
     * Get a payment description for order (shown at Stripe dashboard)
     *
     * @param AbstractOrder $Order
     * @return string
     */
    public static function getPaymentDescriptionForOrder(AbstractOrder $Order): string
    {
        $defaultDescription = $Order->getCustomer()->getLocale()->get(
            'quiqqer/payment-stripe',
            'PaymentIntent.description.default',
            [
                'orderId' => $Order->getPrefixedId()
            ]
        );

        try {
            $template = Provider::getGeneralSetting('payment_description');

            if (empty($template)) {
                return $defaultDescription;
            }

            $placeholders = ['orderId'];

            foreach ($placeholders as $placeholder) {
                switch ($placeholder) {
                    case 'orderId':
                        $replace = $Order->getPrefixedId();
                        break;

                    default:
                        continue 2;
                }

                $template = str_replace('{' . $placeholder . '}', $replace, $template);
            }

            return $template;
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
            return $defaultDescription;
        }
    }

    /**
     * @param Invoice $invoice
     * @param string $stripeInvoiceId
     * @param string $errorMsg
     * @return void
     */
    public static function sendNotificationAboutFailedInvoicePayment(
        Invoice $invoice,
        string $stripeInvoiceId,
        string $errorMsg = ''
    ): void {
        try {
            $locale = QUI::getSystemLocale();
            $mailer = QUI::getMailManager()->getMailer();
            $mailer->addRecipient(QUI::conf('mail', 'admin_mail'));
            $mailer->setSubject(
                $locale->get(
                    'quiqqer/payment-stripe',
                    'mail.payment_failed.invoice.subject',
                    [
                        'invoiceNo' => $invoice->getPrefixedNumber()
                    ]
                )
            );

            $contractNo = '-';
            $contractData = $invoice->getData('contract');

            if (!empty($contractData['prefixedNumber'])) {
                $contractNo = $contractData['prefixedNumber'];
            }

            $mailer->setBody(
                $locale->get(
                    'quiqqer/payment-stripe',
                    'mail.payment_failed.invoice.body',
                    [
                        'invoiceNo' => $invoice->getPrefixedNumber(),
                        'invoiceAmount' => $invoice->getPriceCalculation()->getSum()->formatted($locale),
                        'contractNo' => $contractNo,
                        'stripeInvoiceId' => $stripeInvoiceId,
                        'stripeInvoiceUrl' => self::getStripeDashboardSearchUrl($stripeInvoiceId),
                        'errorMsg' => $errorMsg ?: '-'
                    ]
                )
            );
            $mailer->send();
        } catch (\Exception $exception) {
            QUI\System\Log::writeException($exception);
        }
    }

    /**
     * @param AbstractOrder $order
     * @param string $errorMsg
     * @return void
     */
    public static function sendNotificationAboutFailedOrderPayment(AbstractOrder $order, string $errorMsg = ''): void
    {
        try {
            $locale = QUI::getSystemLocale();
            $mailer = QUI::getMailManager()->getMailer();
            $mailer->addRecipient(QUI::conf('mail', 'admin_mail'));
            $mailer->setSubject(
                $locale->get(
                    'quiqqer/payment-stripe',
                    'mail.payment_failed.order.subject',
                    [
                        'orderId' => $order->getPrefixedNumber()
                    ]
                )
            );

            $paymentIntentId = $order->getPaymentDataEntry(AbstractBasePayment::ATTR_STRIPE_PAYMENT_INTENT_ID);
            $paymentIntentUrl = 'https://dashboard.stripe.com/';

            if (Provider::isSandbox()) {
                $paymentIntentUrl .= 'test/';
            }

            $paymentIntentUrl .= 'payments/' . $paymentIntentId;

            $mailer->setBody(
                $locale->get(
                    'quiqqer/payment-stripe',
                    'mail.payment_failed.order.body',
                    [
                        'orderId' => $order->getPrefixedNumber(),
                        'orderAmount' => $order->getPriceCalculation()->getSum()->formatted($locale),
                        'stripePaymentIntentId' => $order->getPaymentDataEntry(
                            AbstractBasePayment::ATTR_STRIPE_PAYMENT_INTENT_ID
                        ),
                        'paymentIntentUrl' => $paymentIntentUrl,
                        'errorMsg' => $errorMsg ?: '-'
                    ]
                )
            );
            $mailer->send();
        } catch (\Exception $exception) {
            QUI\System\Log::writeException($exception);
        }
    }

    /**
     * @param Dispute $dispute
     * @param OrderInterface|null $order
     * @return void
     */
    public static function sendNotificationAboutDisputedTransaction(
        Dispute $dispute,
        OrderInterface $order = null
    ): void {
        try {
            $locale = QUI::getSystemLocale();
            $mailer = QUI::getMailManager()->getMailer();
            $mailer->addRecipient(QUI::conf('mail', 'admin_mail'));
            $mailer->setSubject(
                $locale->get(
                    'quiqqer/payment-stripe',
                    'mail.disputed_transaction.subject'
                )
            );

            $mailer->setBody(
                $locale->get(
                    'quiqqer/payment-stripe',
                    'mail.disputed_transaction.body',
                    [
                        'amount' => $dispute->amount / 100,
                        'currency' => $dispute->currency,
                        'chargeUrl' => self::getStripeDashboardSearchUrl($dispute->charge),
                        'reason' => $dispute->reason,
                        'orderId' => $order ? $order->getPrefixedNumber() : '-'
                    ]
                )
            );
            $mailer->send();
        } catch (\Exception $exception) {
            QUI\System\Log::writeException($exception);
        }
    }

    /**
     * @param PaymentIntent $paymentIntent
     * @return AbstractOrder|null
     */
    public static function getOrderByPaymentIntent(PaymentIntent $paymentIntent): ?AbstractOrder
    {
        $metaData = $paymentIntent->metadata->toArray();

        if (empty($metaData['orderUuid'])) {
            return null;
        }

        $orderUuid = $metaData['orderUuid'];

        try {
            return OrderHandler::getInstance()->get($orderUuid);
        } catch (\Exception $exception) {
            QUI\System\Log::writeException($exception);
        }

        return null;
    }

    /**
     * Get direct link to Stripe dashboard with an entity id (invoice, dispute, payment intent etc.).
     *
     * @param string $entityId
     * @return string
     */
    public static function getStripeDashboardSearchUrl(string $entityId): string
    {
        $url = 'https://dashboard.stripe.com/';

        if (Provider::isSandbox()) {
            $url .= 'test/';
        }

        $url .= 'search?query=' . $entityId;
        return $url;
    }
}
