<?php

namespace QUI\ERP\Payments\Stripe;

use Exception;
use QUI;
use Stripe\Dispute;
use Stripe\Event as StripeEvent;
use Stripe\PaymentIntent;
use Stripe\Refund as StripeRefund;
use Stripe\Subscription as StripeSubscription;
use QUI\ERP\Accounting\Payments\Transactions\Handler as TransactionsHandler;
use QUI\ERP\Payments\Stripe\PaymentMethods\Recurring\Subscriptions;
use QUI\ERP\Order\Handler as OrderHandler;

/**
 * Class WebhookHandler
 *
 * Handles all relevant Stripe webhook events
 */
class WebhookHandler
{
    /**
     * General entry point for all Stripe webhook events
     *
     * @param StripeEvent $Event
     * @return void
     */
    public static function handleEvent(StripeEvent $Event): void
    {
        switch ($Event->type) {
            case StripeEvent::CHARGE_REFUNDED:
                self::onChargeRefunded($Event);
                break;

            case StripeEvent::CHARGE_REFUND_UPDATED:
                self::onChargeRefundUpdated($Event);
                break;

            case StripeEvent::CUSTOMER_SUBSCRIPTION_DELETED:
                self::onCustomerSubscriptionDeleted($Event);
                break;

            case StripeEvent::CUSTOMER_SUBSCRIPTION_UPDATED:
                self::onCustomerSubscriptionUpdated($Event);
                break;

            case StripeEvent::PAYMENT_INTENT_SUCCEEDED:
            case StripeEvent::PAYMENT_INTENT_CANCELED:
                // currently checked synchronously via cron
//                self::onPaymentIntentEvent($Event);
                break;

            case StripeEvent::CHARGE_DISPUTE_CREATED:
                self::onChargeDisputeCreated($Event);
                break;
        }
    }

    /**
     * Stripe webhook: customer.subscription.deleted
     *
     * @param StripeEvent $Event
     * @return void
     */
    protected static function onCustomerSubscriptionDeleted(StripeEvent $Event): void
    {
        Subscriptions::setSubscriptionAsInactive($Event->data->object->id);
    }

    /**
     * Stripe webhook: customer.subscription.updated
     *
     * @param StripeEvent $Event
     * @return void
     */
    protected static function onCustomerSubscriptionUpdated(StripeEvent $Event): void
    {
        if ($Event->data->object->status !== StripeSubscription::STATUS_ACTIVE) {
            Subscriptions::setSubscriptionAsInactive($Event->data->object->id);
        }
    }

    /**
     * Stripe webhook: charge.refund.updated
     *
     * @param StripeEvent $Event
     * @return void
     */
    protected static function onChargeRefundUpdated(StripeEvent $Event): void
    {
        if (empty($Event->data->object->metadata->refundTxId)) {
            return;
        }

        try {
            $RefundTransaction = TransactionsHandler::getInstance()->get(
                $Event->data->object->metadata->refundTxId
            );

            switch ($Event->data->object->status) {
                case StripeRefund::STATUS_SUCCEEDED:
                    $RefundTransaction->complete();
                    break;

                case StripeRefund::STATUS_CANCELED:
                case StripeRefund::STATUS_FAILED:
                    $RefundTransaction->error();
                    break;
            }
        } catch (Exception $Exception) {
            QUI\System\Log::writeException($Exception);
        }
    }

    /**
     * Stripe webhook: charge.refunded
     *
     * @param StripeEvent $Event
     * @return void
     */
    protected static function onChargeRefunded(StripeEvent $Event): void
    {
        if (empty($Event->data->object->refunds[0]->metadata->refundTxId)) {
            return;
        }

        try {
            $RefundTransaction = TransactionsHandler::getInstance()->get(
                $Event->data->object->refunds[0]->metadata->refundTxId
            );

            if ($RefundTransaction->isPending()) {
                $RefundTransaction->complete();
            }
        } catch (Exception $Exception) {
            QUI\System\Log::writeException($Exception);
        }
    }

    /**
     * Stripe webhook: customer.subscription.deleted
     *
     * @param StripeEvent $event
     * @return void
     */
    protected static function onPaymentIntentEvent(StripeEvent $event): void
    {
        $paymentIntent = $event->data->object;

        if (!($paymentIntent instanceof PaymentIntent)) {
            return;
        }

        $metaData = $paymentIntent->metadata->toArray();

        if (empty($metaData['orderUuid'])) {
            return;
        }

        try {
            $order = OrderHandler::getInstance()->get($metaData['orderUuid']);
        } catch (\Exception $exception) {
            QUI\System\Log::writeDebugException($exception);
            return;
        }

        $orderProcessingService = new OrderProcessingService($order);
        $orderProcessingService->processPaymentIntent($paymentIntent);
    }

    /**
     * Stripe webhook: customer.subscription.deleted
     *
     * @param StripeEvent $event
     * @return void
     */
    protected static function onChargeDisputeCreated(StripeEvent $event): void
    {
        $dispute = $event->data->object;

        if (!($dispute instanceof Dispute)) {
            return;
        }

        try {
            $chargeDisputeService = new ChargeDisputeService();
            $chargeDisputeService->processDispute($dispute);
        } catch (\Exception $exception) {
            QUI\System\Log::writeException($exception);
        }
    }
}
